/* gmpc-lyrdb (GMPC plugin)
 * Copyright (C) 2008-2009 Qball Cow <qball@sarine.nl>
 * Project homepage: http://gmpcwiki.sarine.nl/
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libmpd/debug_printf.h>
#include <gmpc/plugin.h>
#include <gmpc/gmpc_easy_download.h>
#include <gmpc/metadata.h>
#include <glib.h>
#include <config.h>

/**
 * Get/Set enabled 
 */
static int lyrdb_get_enabled()
{
	return cfg_get_single_value_as_int_with_default(config, "lyrdb-plugin", "enable", TRUE);
}
static void lyrdb_set_enabled(int enabled)
{
	cfg_set_single_value_as_int(config, "lyrdb-plugin", "enable", enabled);
}

/* Get priority */
static int lyrdb_fetch_cover_priority(void){
	return cfg_get_single_value_as_int_with_default(config, "lyrdb-plugin", "priority", 80);
}
static void lyrdb_fetch_cover_priority_set(int priority){
	cfg_set_single_value_as_int(config, "lyrdb-plugin", "priority", priority);
}

static int lyrdb_fetch_get_lyric(mpd_Song *song,MetaDataType type, char **path)
{
	int result = 0;
    /* Check if enabled */
	if(lyrdb_get_enabled() == FALSE)
	{
		return META_DATA_UNAVAILABLE;
	}
    /* Can I fetch this type? */
    if(type != META_SONG_TXT)
    {
        return META_DATA_UNAVAILABLE;
    }
    /* Check available fields */
    if(song->artist == NULL || song->title == NULL)
    {
        return META_DATA_UNAVAILABLE;
    }
    gchar *artist, *title;
    int retv = META_DATA_UNAVAILABLE;
    /* escape, for html query */
    artist = g_uri_escape_string(song->artist, NULL, TRUE);
    title =  g_uri_escape_string(song->title,  NULL, TRUE);

    if(artist && title)
    {
        gmpc_easy_download_struct data= {NULL, 0,-1,NULL, NULL};
        /* create the uri */
        gchar *uri_path = g_strdup_printf("http://webservices.lyrdb.com/lookup.php?q=%s%%20%s&for=fullt&agent=gmpc-lyrdb",
                artist, title);
        debug_printf(DEBUG_INFO,"%s\n", uri_path);
        if(gmpc_easy_download(uri_path, &data))
        {
            /* Check error */
            if(data.size > 6 && strncmp(data.data, "error:", 6) == 0 )
            {
                debug_printf(DEBUG_INFO,"Error occured: %s\n", data.data);
            }
            else if( data.size > 6)
            {
                gboolean found = FALSE;
                /* Parse the file */
                gchar **entries = g_strsplit(data.data, "\n", -1);
                if(entries)
                {
                    int i=0;
                    for(i=0;entries[i] && !found;i++)
                    {
                        
                        char **fields = g_strsplit(entries[i],"\\",-1);
                        if(fields)
                        {
                            /* we need 3 fields, last is always NULL, so this should be safe */
                            if( fields[0] && fields[1] && fields[2])
                            {
                                if(strcmp(fields[2], song->artist) == 0)
                                {
                                    debug_printf(DEBUG_INFO," Goint to download lyric: %s\n", fields[0]);
                                    gmpc_easy_download_clean(&data);
                                    g_free(uri_path);
                                    uri_path = g_strdup_printf("http://www.lyrdb.com/getlyr.php?q=%s", fields[0]);
                                    if(gmpc_easy_download(uri_path, &data))
                                    {
                                        if(g_utf8_validate(data.data, data.size,NULL))
                                        {
                                            /* now we want to store it somewhere */
                                            gchar *full_path = gmpc_get_metadata_filename(META_SONG_TXT, song, NULL); 
                                            /* don't need fancy writers/loaders, just dump the file. */
                                            if(g_file_set_contents(full_path, data.data, data.size, NULL))
                                            {
                                                *path = full_path;
                                                retv = META_DATA_AVAILABLE;
                                                found = TRUE;
                                            }
                                            else
                                                g_free(full_path);
                                        }else{
                                            debug_printf(DEBUG_ERROR, "LYRDB returned a non-utf8 lyric, skipping");                                           
                                            found = FALSE;
                                        }
                                    }

                                }
                            }
                            g_strfreev(fields);
                        }
                    }
                    g_strfreev(entries);
                }
            }
            gmpc_easy_download_clean(&data);


        }

        g_free(uri_path);
    }

    if(artist) g_free(artist);
    if(title) g_free(title);
    return retv; 
}	

/**
 * Preferences
 */
static void preferences_enable_cb_callback(GtkToggleButton *button, gpointer data)
{
	lyrdb_set_enabled(gtk_toggle_button_get_active(button));
}
static void preferences_construct(GtkWidget *container)
{
    GtkWidget *label = gtk_label_new("Lyrdb.com Web Services are kindly provided by Flavio González Vázquez");
	GtkWidget *vbox = gtk_vbox_new(FALSE, 6);
	GtkWidget *checkbox = gtk_check_button_new_with_mnemonic("_Enable lyrdb.org as lyric source.");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), lyrdb_get_enabled());

	g_signal_connect(G_OBJECT(checkbox), "toggled", G_CALLBACK(preferences_enable_cb_callback), NULL);
	gtk_box_pack_start(GTK_BOX(vbox), checkbox, FALSE, FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE,0);

	gtk_widget_show_all(vbox);
	gtk_container_add(GTK_CONTAINER(container), vbox);
}

/**
 * Generic de-construct function
 */
static void preferences_destroy(GtkWidget *container)
{
	GtkWidget *widget = gtk_bin_get_child(GTK_BIN(container));
	if(widget)
	{
		gtk_container_remove(GTK_CONTAINER(container),widget);
	}
}

gmpcMetaDataPlugin lw_cover = {
	.get_priority   = lyrdb_fetch_cover_priority,
    .set_priority   = lyrdb_fetch_cover_priority_set,
	.get_image      = lyrdb_fetch_get_lyric
};

gmpcPrefPlugin lw_pref = {
	.construct = preferences_construct,
	.destroy = preferences_destroy
};

int plugin_api_version = PLUGIN_API_VERSION;

gmpcPlugin plugin = {
	.name           = "Lyrdb.com lyric source",
	.version        = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
	.plugin_type    = GMPC_PLUGIN_META_DATA,
	.metadata       = &lw_cover,
	.get_enabled    = lyrdb_get_enabled,
	.set_enabled    = lyrdb_set_enabled,
	.pref 					= &lw_pref
};
